from mayan.apps.appearance.classes import Icon

icon_about = Icon(driver_name='fontawesome', symbol='info')
icon_book = Icon(driver_name='fontawesome', symbol='book')
icon_documentation = Icon(driver_name='fontawesome', symbol='book')
icon_forum = Icon(
    driver_name='fontawesome', symbol='life-ring'
)
icon_license = Icon(
    driver_name='fontawesome', symbol='certificate'
)
icon_menu_about = Icon(
    driver_name='fontawesome', symbol='info'
)
icon_menu_user = Icon(
    driver_name='fontawesome', symbol='user-circle'
)
icon_object_copy = Icon(
    driver_name='fontawesome', symbol='stamp'
)
icon_setup = Icon(
    driver_name='fontawesome', symbol='cog'
)
icon_social_paypal = Icon(
    driver_name='fontawesomecss', css_classes='fab fa-paypal'
)
icon_source_code = Icon(driver_name='fontawesome', symbol='code-branch')
icon_store = Icon(driver_name='fontawesome', symbol='tshirt')
icon_support = Icon(
    driver_name='fontawesome', symbol='phone'
)
icon_tools = Icon(
    driver_name='fontawesome', symbol='wrench'
)
