{% extends 'appearance/base.html' %}

{% load views_tags %}

{% block title %}{% include 'appearance/calculate_form_title.html' with non_html_title=True %}{% endblock %}
{% block content %}
    {% if form %}
        {% with '' as title %}
            {% include 'appearance/generic_form_subtemplate.html' %}
        {% endwith %}
    {% endif %}

    <div class="row">
        {% for subtemplate in subtemplates_list %}
            {% if subtemplate.column_class %}
                <div class="{{ subtemplate.column_class }}">
            {% else %}
                <div class="col-xs-12">
            {% endif %}
                {% views_render_subtemplate subtemplate.name subtemplate.context as rendered_subtemplate %}
                {{ rendered_subtemplate }}
            </div>
        {% endfor %}
    </div>
{% endblock %}
