{% extends 'appearance/base_plain.html' %}

{% load i18n %}
{% load static %}

{% load common_tags %}

{% block base_title %}{% trans 'Password reset' %}{% endblock %}

{% block project_name %}{% endblock %}

{% block content_plain %}
    <div class="row">
        <div class="col-xs-10 col-xs-offset-1 col-sm-8 col-sm-offset-2 col-md-6 col-md-offset-3 col-lg-4 col-lg-offset-4">
            <div class="panel panel-primary">
                <div class="panel-heading">
                    <h3 class="panel-title">&nbsp;</h3>
                </div>
                <div class="panel-body">
                    <h2>{% trans 'Password reset' %}</h2>
                    <hr>
                    <div class="content login">
                        <form action="." method="post" class="form login">{% csrf_token %}
                            <div class="group wat-cf">
                                {% include 'appearance/generic_form_instance.html' %}
                                <input type="hidden" name="next" value="{{ next|escape }}" />

                            <div class="form-group">
                                <button class="btn btn-primary" name="{% if form.prefix %}{{ form.prefix }}-submit{% else %}submit{% endif %}" type="submit"><i class="fa fa-check"></i> {% trans 'Submit' %}</button>
                            </div>
                        </form>
                    </div>
                </div>
            </div>
        </div>
    </div>
{% endblock content_plain %}
