DEFAULT_FIREFOX_GECKODRIVER_PATH = '/usr/bin/geckodriver'

EXCLUDE_TEST_TAG = 'exclude'

TEST_SERVER_HOST = 'testserver'
TEST_SERVER_SCHEME = 'http'
TEST_VIEW_NAME = 'test view name'
TEST_VIEW_URL = 'test-view-url'
