from mayan.apps.dependencies.classes import PythonDependency

PythonDependency(
    module=__name__, name='gevent', version_string='==20.4.0'
)
PythonDependency(
    module=__name__, name='gunicorn', version_string='==20.0.4'
)
PythonDependency(
    module=__name__, name='whitenoise', version_string='==5.0.1'
)
