{% load i18n %}

{% load views_tags %}

{% views_calculate_title as result %}

{% if not non_html_title %}
    <div style="display: flex;">
        <button id="main-menu-button-open">
            <span class="sr-only">{% trans 'Open main menu' %}</span>
            <i aria-hidden="true" data-hidden="true" class="fa fa-bars"></i>
        </button>
{% endif %}

{% if not non_html_title and result.title %}
        <div>
            <h3 id="content-title" title='{{ result.title_full }}'>
{% endif %}

                {{ result.title }}

{% if not non_html_title and result.title %}
            </h3>
            {% if subtitle %}
                <p>
                    <small>{{ subtitle }}</small>
                </p>
            {% endif %}
{% endif %}

{% if not non_html_title %}
        </div>
    </div>
    {% if title or subtitle %}<hr/>{% endif %}
{% endif %}
