from mayan.apps.dependencies.classes import PythonDependency

PythonDependency(
    module=__name__, name='extract-msg', version_string='==0.23.3'
)
PythonDependency(
    module=__name__, name='pycryptodome', version_string='==3.9.7'
)
